
-- global saved vars
MoneyTrailDB = {} -- will be overwritten during variables loaded

-- locals
local realmName = GetRealmName()
local playerName = UnitName("player")
local playerClass = select(2, UnitClass("player"))
local data -- will contain the data we work with 
local displays = {} -- all our displays
local HookDisplays -- func defined later

-- localization

local L = {
	total = "Total"
}

-- local funcs

local classColors = {
	WTF = "|cffa0a0a0"
}
for k, v in pairs(RAID_CLASS_COLORS) do
	classColors[k] = "|cff" .. string.format("%02x%02x%02x", v.r * 255, v.g * 255, v.b * 255)
end

local coloredNames = setmetatable({}, {__index =
	function(self, key)
		if type(key) == "nil" then return nil end
		local class = MoneyTrailDB[realmName][key] and MoneyTrailDB[realmName][key].class
		if class then
			self[key] = classColors[class]  .. key .. "|r"
			return self[key]
		else
			return classColors.WTF .. key .. "|r"
		end
	end
})

local function MoneyString( money )
	local gold = abs(money / 10000)
	local silver = abs(mod(money / 100, 100))
	local copper = abs(mod(money, 100))
	
	if money > 10000 then
		return string.format( "|cffffffff%d|r|cffffd700g|r |cffffffff%d|r|cffc7c7cfs|r |cffffffff%d|r|cffeda55fc|r", gold, silver, copper)
	elseif money > 100 then
		return string.format( "|cffffffff%d|r|cffc7c7cfs|r |cffffffff%d|r|cffeda55fc|r", silver, copper)	
	else 
		return string.format("|cffffffff%d|r|cffeda55fc|r", copper )
	end
end

-- the most important func
local function UpdateData()
	local money = GetMoney()
	if money ~= data.money then
		-- money changed calculate that shit
		if not data.money then data.money = 0 end -- init 0
		data.gainSession = money - data.money
		data.money = money
	end
end

-- our addon frame
local addon = CreateFrame("Frame", "MoneyTrail", UIParent)
-- our addon event handler
local function OnEvent(self, event, ...)
	if self[event] then
		self[event](self, event, ...)
	end
end

function addon:VARIABLES_LOADED()
	-- intialize anything we might need to init
	MoneyTrailDB[realmName] = MoneyTrailDB[realmName] or {}
	MoneyTrailDB[realmName][playerName] = MoneyTrailDB[playerName] or {}
	MoneyTrailDB[realmName][playerName].class = playerClass
	data = MoneyTrailDB[realmName][playerName]
end

function addon:PLAYER_LOGOUT()
	-- write saved vars
	MoneyTrailDB[realmName][playerName] = data
end

function addon:PLAYER_LOGIN()
	-- setup our data
	UpdateData()
end

-- PEW is late enough for all addons OnEnables to have been called
function addon:PLAYER_ENTERING_WORLD()
	addon:UnregisterEvent("PLAYER_ENTERING_WORLD")
	HookDisplays()
end

-- register our events
addon:RegisterEvent("VARIABLES_LOADED")
addon:RegisterEvent("PLAYER_LOGOUT")
addon:RegisterEvent("PLAYER_LOGIN")
addon:RegisterEvent("PLAYER_ENTERING_WORLD")
addon:SetScript("OnEvent", OnEvent)


-- get our display set up
local function OnEnter(self)
	UpdateData()
	
	GameTooltip:SetOwner(self.MoneyTrailAnchor and self.MoneyTrailAnchor or self, 'ANCHOR_TOPRIGHT')
	
	GameTooltip:AddLine("MoneyTrail")
	local total = 0
	for pn, d in pairs(MoneyTrailDB[realmName]) do
		total = total + d.money
		GameTooltip:AddDoubleLine(coloredNames[pn], MoneyString(d.money))
	end
	GameTooltip:AddDoubleLine(L.total, MoneyString(total))
	
	GameTooltip:Show()
end

local function OnLeave(self)
	GameTooltip:Hide()
end

function HookDisplays()
	-- simple bagframes support
	-- true means multiple frames anr reanchoring
	-- false means single frame
	local BagFrames = {
		["ContainerFrame1MoneyFrame"] = true, -- Blizzard Backpack
		["OneBagFrameMoneyFrame"] = true, -- OneBag
		["BagginsMoneyFrame"] = false, -- Baggins
	}
	for frame, multiple in pairs(BagFrames) do
		if _G[frame] then
			table.insert(displays, _G[frame])
			if multiple then
				table.insert(displays, _G[frame.."CopperButton"])
				table.insert(displays, _G[frame.."SilverButton"])
				table.insert(displays, _G[frame.."GoldButton"])	
				_G[frame.."CopperButton"].MoneyTrailAnchor = _G[frame]
				_G[frame.."SilverButton"].MoneyTrailAnchor = _G[frame]
				_G[frame.."GoldButton"].MoneyTrailAnchor = _G[frame]
			end
		end
	end

	-- Combuctor Support, it has a nameless frame with OnEnter etc so we specialcase this
	if _G["CombuctorFrame1MoneyFrame"] then
		local kids = { _G["CombuctorFrame1MoneyFrame"]:GetChildren() }
		for k, child in pairs(kids) do
			if child:HasScript("OnEnter") and child:GetScript("OnEnter") then
				table.insert(displays, child)
			end
		end
	end

	-- 'Hook' the displays
	for k, frame in pairs(displays) do
		frame:EnableMouse(true)
		if frame:GetScript("OnEnter") then frame:HookScript("OnEnter", OnEnter)
		else frame:SetScript("OnEnter", OnEnter) end
		
		if frame:GetScript("OnLeave") then frame:HookScript("OnLeave", OnLeave)
		else frame:SetScript("OnLeave", OnLeave) end
	end
end
